""" Unit tests of curate_registry_utils
"""

from unittest.case import TestCase

from core_curate_registry_app.utils import jquery
from core_parser_app.components.data_structure_element.models import DataStructureElement


class TestUtilsGetJqueryFromDataStructureElement(TestCase):

    def test_utils_get_jquery_from_data_structure_element_returns_data(self):
        # Arrange
        string_expected = ".element1,.element2,.element3"
        object1 = "element1"
        object2 = "element2"
        object3 = "element3"
        data_list = [object1, object2, object3]
        # Act
        result = jquery.get_jquery_selector_from_data_structure(data_list)
        # Assert
        self.assertEqual(string_expected, result)

    def test_utils_get_jquery_from_data_structure_element_returns_empty(self):
        # Arrange
        data_list = []
        string_expected = ""
        # Act
        result = jquery.get_jquery_selector_from_data_structure(data_list)
        # Assert
        self.assertEqual(string_expected, result)

    def test_utils_get_jquery_from_data_structure_element_returns_one_element(self):
        # Arrange
        object1 = "element1"
        string_expected = ".element1"
        data_list =[object1]
        # Act
        result = jquery.get_jquery_selector_from_data_structure(data_list)
        # Assert
        self.assertEqual(string_expected, result)




"""Data util for curate_registry app
"""


def get_jquery_selector_from_data_structure(data_id_list):
    """Create a jquery_selector from xpath
    Args : List of ObjectId from data structure elements
    Returns: Jquery_selector from xpath (.ID1,.ID2,.ID3 ...)

    """

    jquery_selector = ""
    for data_id in data_id_list:
        jquery_selector = jquery_selector + ("." + str(data_id) + ",")
    jquery_selector = jquery_selector[:-1]
    return jquery_selector



""" Curate Data Structure registry api
"""
from core_main_app.utils import xml as xml_utils

from core_main_registry_app.utils.role.extraction import role_extraction


def get_role(curate_data_structure):
    """ Get the role saved in the curate_data_structure's form string

    Args:
        curate_data_structure:

    Returns:

    """
    return role_extraction(xml_utils.raw_xml_to_dict(curate_data_structure.form_string, xml_utils.post_processor))



SECRET_KEY = 'fake-key'

INSTALLED_APPS = [
    # Django apps
    'django.contrib.admin',
    'django.contrib.auth',
    'django.contrib.contenttypes',
    'django.contrib.sites',

    # Local apps
    "tests",
]



"""Curate registry app user ajax
"""

import core_curate_app.permissions.rights as rights
import core_main_app.utils.decorators as decorators
from core_curate_app.views.user import ajax as curate_ajax


@decorators.permission_required(content_type=rights.curate_content_type,
                                permission=rights.curate_access, raise_exception=True)
def start_curate(request):
    """ Load forms to start curating.
    Add role to response url when POST.

    Args:
        request:

    Returns:

    """
    if request.method == 'GET':
        return curate_ajax.start_curate(request)
    elif request.method == 'POST':
        response = curate_ajax.start_curate(request)
        if response.status_code == 200:
            role = request.GET.get('role', None)
            response.content = "{0}?role={1}".format(response.content.decode('utf-8'), role)
        return response



""" Url router for the curate application
"""
from django.conf.urls import url
from django.core.urlresolvers import reverse_lazy

import core_curate_app.permissions.rights as rights
import core_curate_app.views.user.ajax as user_ajax
import core_curate_app.views.user.views as user_views
import core_curate_registry_app.views.user.ajax as user_registry_ajax
import core_curate_registry_app.views.user.views as user_registry_views
from core_curate_app.views.common import views as common_views
from core_main_app.utils.decorators import permission_required

urlpatterns = [
    url(r'^$', user_registry_views.index,
        name='core_curate_index'),
    url(r'^start_curate$', user_registry_ajax.start_curate,
        name='core_curate_start'),
    url(r'^start_curate/(?P<role>[\w-]+)/$', user_registry_views.start_curate,
        name='core_curate_start_from_role'),
    url(r'^enter-data/(?P<curate_data_structure_id>\w+)$', user_registry_views.EnterDataRegistryView.as_view(),
        name='core_curate_enter_data'),
    # FIXME: url to allow reopening a form with unsaved changes (may be temporary until curate workflow redesign)
    url(r'^enter-data/(?P<curate_data_structure_id>\w+)/(?P<reload_unsaved_changes>\w+)$',
        user_registry_views.EnterDataRegistryView.as_view(),
        name='core_curate_enter_data'),
    url(r'^view-data/(?P<curate_data_structure_id>\w+)$', user_registry_views.ViewDataRegistryView.as_view(),
        name='core_curate_view_data'),
    url(r'^download-xml/(?P<curate_data_structure_id>\w+)$', user_views.download_current_xml,
        name='core_curate_download_xml'),
    url(r'^download-xsd/(?P<curate_data_structure_id>\w+)$', user_views.download_xsd,
        name='core_curate_download_xsd'),
    url(r'^generate-choice/(?P<curate_data_structure_id>\w+)$', user_ajax.generate_choice,
        name='core_curate_generate_choice'),
    url(r'^generate-element/(?P<curate_data_structure_id>\w+)$', user_ajax.generate_element,
        name='core_curate_generate_element'),
    url(r'^remove-element$', user_ajax.remove_element,
        name='core_curate_remove_element'),
    url(r'^clear-fields$', user_ajax.clear_fields,
        name='core_curate_clear_fields'),
    url(r'^cancel-changes$', user_ajax.cancel_changes,
        name='core_curate_cancel_changes'),
    url(r'^cancel-form$', user_ajax.cancel_form,
        name='core_curate_cancel_form'),
    url(r'^save-form$', user_ajax.save_form,
        name='core_curate_save_form'),
    url(r'^save-data$', user_ajax.save_data,
        name='core_curate_save_data'),
    url(r'^validate-form$', user_ajax.validate_form,
        name='core_curate_validate_form'),
    url(r'^view-form/(?P<curate_data_structure_id>\w+)$',
        permission_required(content_type=rights.curate_content_type,
                            permission=rights.curate_access,
                            login_url=reverse_lazy("core_main_app_login"))(common_views.FormView.as_view()),
        name='core_curate_view_form'),

]



"""Core Curate App Settings
"""
import os

from django.conf import settings

if not settings.configured:
    settings.configure()

REGISTRY_XSD_FILENAME = getattr(settings, 'REGISTRY_XSD_FILENAME', "")
""" str: Registry xsd filename used for the initialisation.
"""


BASE_DIR = os.path.dirname(os.path.dirname(os.path.abspath(__file__)))
LOCALE_PATHS = (
    os.path.join(BASE_DIR, 'core_curate_registry_app/locale'),
)


XPATH_TITLE = "/rsm:Resource[1]/rsm:identity[1]/rsm:title"
""" str : Xpath of the resource name/title 
"""